package rule

import (
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/db"
	zhios_order_relate_utils "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/utils"
	"errors"
	"github.com/shopspring/decimal"
	"strconv"
	"time"
	"xorm.io/xorm"
)

/*
EstimateUserPosition 预估用户位置
total 排名总数
level 预估排名层级
levelRank  用户当前层级排名
rank  用户总排名
previousRow  上一层级
diffValue  距离上一层级差值
*/
func EstimateUserPosition(engine *xorm.Engine, uid int) (total, level, levelRank, rank, previousRow int, diffValue string, err error) {
	//1、查找 `user_public_platoon_setting` 基础设置
	userPublicPlatoonSetting, err := db.UserPublicPlatoonSettingGetOneByParams(engine, map[string]interface{}{
		"key":   "is_open",
		"value": 1,
	})
	if err != nil {
		return
	}
	if userPublicPlatoonSetting == nil {
		return
	}
	//2、查询排名
	rank, total, userAmount, err := CalcUserRank(engine, zhios_order_relate_utils.IntToStr(uid))
	if err != nil {
		return
	}
	rows := float64(userPublicPlatoonSetting.SeveralRows)
	times := float64(userPublicPlatoonSetting.SeveralTimes)
	level = makeSearchLevelByPosition(&rank, rows, &times)
	levelPosition1 := getLevelForFirstPosition(level, userPublicPlatoonSetting.SeveralTimes)
	levelRank = rank - levelPosition1 + 1

	//3、计算与前排差距
	previousRow = level - 1
	if previousRow > 0 {
		previousRowPosition1 := getLevelForLastPosition(previousRow, userPublicPlatoonSetting.SeveralTimes)
		previousRowAmount, err1 := GetUserRankAmount(engine, strconv.Itoa(previousRowPosition1))
		if err1 != nil {
			err = err1
			return
		}
		userAmountValue, _ := decimal.NewFromString(userAmount)
		previousRowAmountValue, _ := decimal.NewFromString(previousRowAmount)
		diffValue = previousRowAmountValue.Sub(userAmountValue).String()
	}
	return
}

func CalcUserRank(engine *xorm.Engine, uid string) (rank, total int, amount string, err error) {
	sql := "SELECT id, uid, amount, @rank := @rank + 1 AS rank FROM `user_public_platoon_double_network_user_coin_record`, (SELECT @rank:=0) r ORDER BY amount DESC;"
	nativeString, _ := db.QueryNativeString(engine, sql)
	total = len(nativeString)
	if total <= 0 {
		err = errors.New("当前无排名数据")
		return
	}
	for _, v := range nativeString {
		if uid == v["uid"] {
			rank = zhios_order_relate_utils.StrToInt(v["rank"])
			amount = v["amount"]
			break
		}
	}
	return
}

func GetUserRankAmount(engine *xorm.Engine, rank string) (amount string, err error) {
	sql := "SELECT id, uid, amount, @rank := @rank + 1 AS rank FROM `user_public_platoon_double_network_user_coin_record`, (SELECT @rank:=0) r ORDER BY amount DESC;"
	nativeString, _ := db.QueryNativeString(engine, sql)
	if len(nativeString) <= 0 {
		err = errors.New("当前无排名数据")
		return
	}
	for _, v := range nativeString {
		if rank == v["rank"] {
			amount = v["amount"]
			break
		}
	}
	return
}

func ResetPublicPlatoonDoubleNetwork(engine *xorm.Engine, dbName string) (err error) {
	//1、查找 `user_public_platoon_double_network_setting` 基础设置
	now := time.Now()
	userPublicPlatoonDoubleNetworkSetting, err := db.UserPublicPlatoonDoubleNetworkSettingGetOneByParams(engine, map[string]interface{}{
		"key":   "is_open",
		"value": 1,
	})
	if err != nil {
		return err
	}
	if userPublicPlatoonDoubleNetworkSetting.IsOpen != 1 {
		return errors.New("未开启公排双网！")
	}
	if userPublicPlatoonDoubleNetworkSetting.SettlementDate != now.Format("2006-01-02") {
		return errors.New("非重置日期！")
	}

	//2、计算排名数据
	sql := "SELECT id, uid, amount, @rank := @rank + 1 AS rank FROM `user_public_platoon_double_network_user_coin_record`, (SELECT @rank:=0) r ORDER BY amount DESC;"
	nativeString, _ := db.QueryNativeString(engine, sql)
	if len(nativeString) <= 0 {
		return errors.New("当前无排名数据")
	}
	//for _, v := range nativeString {
	//	//if rank == v["rank"] {
	//	//	amount = v["amount"]
	//	//	break
	//	//}
	//}
	return
}
